part of masamune;

class ValidateContext {
  ValidateContext._(this.validationIds);
  final List<String> validationIds;
  final Set<String> _validatedIds = {};

  void set(String id) {
    if (_validatedIds.contains(id)) {
      return;
    }
    _validatedIds.add(id);
  }

  void unset(String id) {
    _validatedIds.remove(id);
  }

  bool get validated {
    for (final id in validationIds) {
      if (!_validatedIds.contains(id)) {
        return false;
      }
    }
    return true;
  }

  void _dispose() {
    _validatedIds.clear();
  }
}

extension WidgetRefValidatorExtensions on WidgetRef {
  void useValidator(List<String> validationIds) {
    valueBuilder<ValidateContext, _ValidatorValue>(
      key: "validator_${validationIds.join(",")}",
      builder: () {
        return _ValidatorValue(validationIds);
      },
    );
  }
}

class _ValidatorValue extends ScopedValue<ValidateContext> {
  const _ValidatorValue(this.validationIds);
  final List<String> validationIds;
  @override
  ScopedValueState<ValidateContext, ScopedValue<ValidateContext>>
      createState() => _ValidatorValueState();
}

class _ValidatorValueState
    extends ScopedValueState<ValidateContext, _ValidatorValue> {
  late final ValidateContext _validator;

  @override
  void initValue() {
    super.initValue();
    _validator = ValidateContext._(value.validationIds);
  }

  @override
  ValidateContext build() => _validator;

  @override
  void dispose() {
    super.dispose();
    _validator._dispose();
  }
}
